@@ -23,10 +23,10 @@ copyright = '2020, The Firebird Project'
 author = 'Pavel Císař'
 
 # The short X.Y version
-version = '0.5.0'
+version = '1.0.0'
 
 # The full version, including alpha/beta/rc tags
-release = '0.5.0'
+release = '1.0.0'
 
 
 # -- General configuration ---------------------------------------------------
@@ -55,7 +55,7 @@ source_suffix = '.txt'
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
 # This pattern also affects html_static_path and html_extra_path.
-exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
+exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store', 'requirements.txt']
 
 default_role = 'py:obj'
 
