@@ -32,10 +32,31 @@
 #                 ______________________________________.
 
 """Firebird Base - Various collection types
+
+This module provides data structures that behave much like builtin `list` and `dict` types,
+but with direct support of operations that can use structured data stored in container, and
+which would normally require utilization of `operator`, `functools` or other means.
+
+All containers provide next operations:
+
+* `filter` and `filterfalse` that return generator that yields items for which `expr` is
+  evaluated as True (or False).
+* `find` that returns first item for which `expr` is evaluated as True, or default.
+* `contains` that returns True if there is any item for which `expr` is evaluated as True.
+* `occurrence` that returns number of items for which `expr` is evaluated as True.
+* `all` and `any` that return True if `expr` is evaluated as True for all or any list element(s).
+* `report` that returns generator that yields data produced by expression(s) evaluated on
+  list items.
+
+Individual collection types provide additional operations like splitting and extracting
+based on expression etc.
+
+Expressions used by these methods could be strings that contain Python expression
+referencing the collection item(s), or lambda functions.
 """
 
 from __future__ import annotations
-from typing import Type, Union, Any, Dict, List, Tuple, Mapping, Sequence, Generator, \
+from typing import T, Type, Union, Any, Dict, List, Tuple, Mapping, Sequence, Generator, \
      Iterable, Callable, cast
 from operator import attrgetter
 import copy
@@ -203,12 +224,9 @@ Example:
                 return True
         return False
 
-class DataList(list, BaseObjectCollection):
+class DataList(List[T], BaseObjectCollection):
     """List of data (objects) with additional functionality.
-"""
-    def __init__(self, items: Iterable=None, type_spec: TypeSpec=UNDEFINED,
-                 key_expr: str=None, frozen: bool=False):
-        """
+
 Arguments:
     items:     Sequence to initialize the collection.
     type_spec: Reject instances that are not instances of specified types.
@@ -221,6 +239,8 @@ Arguments:
 Raises:
     ValueError: When initialization sequence contains invalid instance.
 """
+    def __init__(self, items: Iterable=None, type_spec: TypeSpec=UNDEFINED,
+                 key_expr: str=None, frozen: bool=False):
         assert key_expr is None or isinstance(key_expr, str)
         assert key_expr is None or make_lambda(key_expr) is not None
         if items is not None:
@@ -346,13 +366,14 @@ It's not possible to `add`, `delete` or `change` items in frozen list, but `.sor
         if self.__key_expr:
             fce = make_lambda(self.__key_expr)
             self.__map = dict(((key, index) for index, key in enumerate((fce(item) for item in self))))
-    def split(self, expr: FilterExpr) -> Tuple['DataList', 'DataList']:
+    def split(self, expr: FilterExpr, frozen: bool=False) -> Tuple[DataList, DataList]:
         """Return two new `DataList` instances, first with items for which `expr` is
 evaluated as True and second for `expr` evaluated as False.
 
 Arguments:
     expr: A callable accepting one parameter or expression as string referencing list
           item as `item`.
+    frozen: Create frozen lists.
 
 Example:
     .. code-block:: python
@@ -360,17 +381,18 @@ Example:
        below, above = L.split(lambda x: x.size > 100)
        below, above = L.split('item.size > 100')
 """
-        return DataList(self.filter(expr), self._type_spec, self.__key_expr), \
-               DataList(self.filterfalse(expr), self._type_spec, self.__key_expr)
-    def extract(self, expr: FilterExpr):
-        """Move items for which `expr` is evaluated as True into new `DataList`.
+        return DataList(self.filter(expr), self._type_spec, self.__key_expr, frozen=frozen), \
+               DataList(self.filterfalse(expr), self._type_spec, self.__key_expr, frozen=frozen)
+    def extract(self, expr: FilterExpr, copy: bool=False) -> DataList:
+        """Move/copy items for which `expr` is evaluated as True into new `DataList`.
 
 Arguments:
     expr: A callable accepting one parameter or expression as string referencing list item
           as `item`.
+    copy: When True, items are not removed from source DataList.
 
 Raises:
-    TypeError: When list is frozen
+    TypeError: When list is frozen and `copy` is False.
 
 Example:
     .. code-block:: python
@@ -378,7 +400,8 @@ Example:
        L.extract(lambda x: x.name.startswith("ABC"))
        L.extract('item.name.startswith("ABC")')
 """
-        self.__updchk()
+        if not copy:
+            self.__updchk()
         fce = expr if callable(expr) else make_lambda(expr)
         l = DataList(type_spec=self._type_spec, key_expr=self.__key_expr)
         i = 0
@@ -386,7 +409,10 @@ Example:
             item = self[i]
             if fce(item):
                 l.append(item)
-                del self[i]
+                if not copy:
+                    del self[i]
+                else:
+                    i += 1
             else:
                 i += 1
         return l
@@ -436,6 +462,9 @@ Example:
 class Registry(BaseObjectCollection, Mapping[Any, Distinct]):
     """Mapping container for `.Distinct` objects.
 
+Arguments:
+    data: Either a `.Distinct` instance, or sequence or mapping of `.Distinct` instances.
+
 Any method that expects a `key` also acepts `.Distinct` instance.
 
 To store items into registry with existence check, use:
@@ -463,11 +492,6 @@ Whenever a `key` is required, you can use either a `Distinct` instance, or any v
 represens a key value for instances of stored type.
 """
     def __init__(self, data: Union[Mapping, Sequence, Registry]=None):
-        """Registry initialization.
-
-Arguments:
-    data: Either a `.Distinct` instance, or sequence or mapping of `.Distinct` instances.
-"""
         self._reg: Dict = {}
         if data:
             self.update(data)
